#!/bin/bash
#
# landisk-update.sh
#
# $Id$
#

PATH=/usr/local/sbin:/usr/local/bin:/sbin:/bin:/usr/sbin:/usr/bin

. /mnt/hda5/.c4/data/MODEL
MODEL="${model%
}"	# CRコード除去
CURRENTVERSION="${version%
}"	# CRコード除去

ROOTPATH='/mnt/hda5/.update'

VERFILE="${ROOTPATH}/for_${MODEL}_series"
VEROUTFILE="/mnt/hda5/.c4/data/MODEL"
UPFILE="/boot/.update/update.tgz"
BOOTFILE=`ls -1 ${ROOTPATH}/lepus_linux_SL0AP_*.bin | tail -n 1 | tr -d '\n'`
TARFILE="${ROOTPATH}/update-files.tar.gz"
BETAFILE="${ROOTPATH}/beta"
TMPFILE='/tmp/.landisk-update.sh.tmp'
LOCKDIR='/var/c4/tmp/lock'
CONFTAR="${ROOTPATH}/confupdate.tgz"
CONFDIR='/mnt/hda5/.c4/data/setting/'
SMBCONFFILE='/etc/samba/smb.conf'

UPDATEVER=`sed 's/-.*$//' $VERFILE`
[ -f $BETAFILE ] && BETA=`cat $BETAFILE`

start_led() {
   /usr/local/bin/ledcont progress
}

exit_select() {
#    /usr/local/bin/ledcont errfastblink
#    /usr/local/bin/buzcont 10 50 3
    rm -rf $LOCKDIR
    exit $1
}

###############################################################################
# conffilesの更新
###############################################################################
__update_conffiles() {
	local f=''
	local verup_files='ddns remotelink mywebserver'

	# conffilesの追加
	for f in $verup_files
	do
		if [ ! -e $CONFDIR/$f ]; then
			tar xpzf ${CONFTAR} \
			-C /mnt/hda5 .c4/data/setting/$f ||return 1
		fi
	done
}

###############################################################################
# website.conf が存在しなければtouch する
###############################################################################
__website_conf() {
	local website_conf='/etc/apache2/website.conf'

	# conffilesの追加
	if [ ! -e $website_conf ]; then
		touch $website_conf ||return 1
	fi
	return 0
}

###############################################################################
# ブートローダの比較
###############################################################################
__cmp_boot() {
	local slboot="$1"
	local size=`wc -c < $slboot`
	local oldboot="/tmp/oldboot.bin"
	dd if=/dev/rom0 bs=64k | dd of=$oldboot bs=1 count=$size
	cmp -s $oldboot $slboot
	local status=$?
	rm -f $oldboot
	return $status
}

###############################################################################
# ブートローダのアップデート
###############################################################################
__update_boot() {
	local slboot="$1"
	[ "$slboot" = '' ] && return 0
	echo --- update SL-boot
	if __cmp_boot "$slboot"; then
		echo --- same version. not update.
		return 0
	fi
	echo -n --- different version. update now ...
	dd if=$slboot of=/dev/rom0 bs=64k || return 1
	echo done.
	blockdev --flushbufs /dev/rom0
	echo --- compare
	__cmp_boot "$slboot" && return 0
	echo === error
	return 1
}

########################
# start
start_led

# perlでアップデート
# ${UPPERL} --update
# if [ $? != 0 ]; then
#    exit_select 3
# fi

# サービスの停止の布石
# SERVICE=`${UPPERL} --getservice 2>/dev/null | awk -F '=' '{print $1}'`
# remount rw /
mount -o remount,rw,noatime /
mount -o remount,rw,noatime /boot

# サービスの停止
for i in apache2 samba slpd netatalk mt-daapd dlna avahi-daemon HDLFind.sh cron networking tms webaccess
do
    /etc/init.d/$i stop
done

# mt-daapdはDB構築中などの際/etc/init.d/mt-daapd stopでは停止しないため確実にkillする
if pidof mt-daapd >/dev/null ;then
	echo 'killing mt-daapd...'
	killall -KILL /usr/sbin/mt-daapd
	# 完全にプロセスが死ぬのを2秒待機
	sleep 2
	echo 'done.'
fi

# 必要であれば
#killall buttond
#killall usbhdmng

# ブートローダの書き換え
__update_boot ${BOOTFILE} || exit_select 1

# mt-daapd不要ファイルの削除
rm -f /usr/share/mt-daapd/admin-root/aspl-license.html
rm -f /usr/share/mt-daapd/admin-root/aspl-license.txt
rm -f /usr/share/mt-daapd/admin-root/gpl-license.html
rm -f /usr/share/mt-daapd/admin-root/gpl-license.txt
rm -f /usr/share/mt-daapd/admin-root/mt-daapd.css
rm -f /usr/share/mt-daapd/admin-root/mt-daapd.png
rm -f /usr/share/mt-daapd/admin-root/status.html
rm -f /etc/mt-daapd.conf
rm -f /etc/mt-daapd.playlist
rm -f /etc/mt-daapd/mt-daapd.conf.OLD

# HDLFind不要ファイルの削除
rm -f /usr/local/bin/HDLfind_set.sh
rm -f /etc/rc0.d/K99HDLFind.sh
rm -f /etc/rc6.d/K99HDLFind.sh

# 1.05でmt-daapd入れ替えのため、1.04以前はデータベース削除
case "${CURRENTVERSION}" in
	'1.00'|'1.01'|'1.02'|'1.03'|'1.04')
		echo 'removing songs.gdb...'
		rm -rvf '/mnt/hda5/mt-daapd/'*
		echo 'done.'
		;;
	*) ;;
esac

# tarで展開
tar xpzf ${TARFILE} -C / --exclude=./mnt/hda5/webaccess/smbweb.ini \
			--exclude=./mnt/hda5/webaccess/sharelst.txt \
			--exclude=./mnt/hda5/tms/tms.ini \
			|| exit_select 1
# keep old files
tar xpkzf ${TARFILE} -C / ./mnt/hda5/webaccess/smbweb.ini \
			./mnt/hda5/webaccess/sharelst.txt \
			./mnt/hda5/tms/tms.ini

__update_conffiles || exit_select 1
__website_conf || exit_select 1

# smb.confの編集
grep 'local master' $SMBCONFFILE
[ $? -eq 0 ] && sed -i '/^\s*local master\s*=/d' $SMBCONFFILE

# ファイルを削除する
rm -f $TARFILE $BETAFILE $VERFILE $CONFTAR

# 終了処理
upver="$UPDATEVER"
[ "$UPDATESUBVER" != "" ] && upver="$upver-$UPDATESUBVER"
[ "x$BETA" != "x" ] && upver="$upver beta$BETA"

# version編集
echo "model=$MODEL" > $TMPFILE
echo "version=$upver" >> $TMPFILE
mv $TMPFILE $VEROUTFILE || exit_select 2
chown root:root $VEROUTFILE
chmod 777 $VEROUTFILE
echo "UPDATE->$upver" >/dev/console
echo "success($UPDATEVER)" > /mnt/hda5/.update/update.log
